/*My includes*/

/* Debugging */
//#include "prettyprint.hpp"
#ifdef DEBUG_BUILD
#define DBG(x)		do{if(1){std::cerr<<#x<<":\t["<<x<<"]"<<std::endl;}}while(0);
#else
#define DBG(x)		;
#endif

//#include <QCoreApplication>
#include "Robot.hpp"
#include "Simulator.hpp"
#include <iostream>
#include <csignal>
#include <unistd.h>

/* Signal Handler */
namespace{
	volatile std::sig_atomic_t gSignalStatus;
}
void signal_handler(int signal){
	gSignalStatus=signal;
}

extern "C" {
	#include "extApi.h"
	#include "v_repLib.h"
}
int main(int argc, char *argv[]){
	//std::cin.sync_with_stdio(false);
	//std::ios::sync_with_stdio(false);
	DBG("Debugging is ON");
	
	std::signal(SIGINT, signal_handler);
	
	Robot *robot;
	Simulator *vrep = new Simulator("127.0.0.1", 25000);
	
	for(;;){
		try{
			if (vrep->connect() ==-1){
				std::cout << "Failed to Connect" << std::endl;
				return 0;
			}
			break;
		}catch(...){
			std::cout<<"Could not connect. Retrying..."<<std::endl;
		}
	}
	
	robot = new Robot(vrep, "Pioneer_p3dx");
	
	/*Main Loop*/
	
	static bool ad_infinitum=true;
	static unsigned int i_max=3000*56;
	static unsigned int sleep_us=200;
	
	for (int i=0;(gSignalStatus!=SIGINT) && (ad_infinitum||i<i_max);++i){
		robot->updateSensors();
		//robot->writeGT();
		//robot->writeSonars();
		//extApi_sleepMs(50);
		usleep(sleep_us);
	}
	std::cout<<std::endl<<"Disconnecting..."<<std::endl;
	vrep->disconnect();
	exit(0);
}
